/*
 * Copyright 2018-2023, ttcoder of TTS-LLC
 * All rights reserved. Distributed under the terms of the MIT license.
 */
#ifndef _WindowNitpickerPriv_H
#define _WindowNitpickerPriv_H


// libstdc++
#include <queue>  // member

// genode
//#include <base/entrypoint.h>

// haiku.lib.so
#include "_CanvasGenode.h"  // member
#include <app/Application.h>


namespace hog {
	//class MouseState;
	class InputBridger;
	class Wm;
	
	///ToDo: create "_GeInput.h" and move these there:
	namespace conv_input {
		int HaikuKey_for_GeKey(
			const	Input::Keycode genode_scancode,
					int32 & changed_disambiguated_modifier
			);
		int ModifiedKey_for_RawBeKey(
			int raw_be_key,
			int virtualized_modifiers
			);
	}
	
	//+ NitpickerInternals !
}



//#pragma mark


struct NitpickerInternals
{
public:

		NitpickerInternals( BBitmap * paint_offscreen = NULL );
		
		CanvasGenode & Canvas(); //+ "PaintingSurface" ?
		Input::Session_client & Input();
		
		void Create_Bare_Window();
		void Destroy_Bare_Window();
		void Set_Title( const char * title );
		void To_Front();
		void To_Back();
		void Background();
		void SetClientGeometry( Gui::Rect inner_rect );
			///++ "SetInnerGeometry( Rect inner_rect ) !
		
#if defined( HAS_MICRO_WINDOW_MANAGER )
		Gui::Connection & ConnectionAndGeom();//nah!: ///+ "DecoratedGeom" ?
		void Set_ClientSide_Decorations(
			BRect inner_frame,
			BRect tab_frame,
			float border_thickness
			);
#endif


private:  // Code

		void reGeom(   Gui::Rect inner_rect );
		void reBuffer( Gui::Rect inner_rect );


private:  // Data

		Gui::Connection //*
			nitpicker;
		CanvasGenode //*
			geCanvas;
		
		// We'll call create_view() and destroy_view() on the session:
//actually it works just as well to call the Connection above
//instead of this Session! Delete this?:
//		Nitpicker::Session_client &
//			_nitpicker;  //+++ managerWindow
		// A window handle is named a * "view" * handle in NitPicker:
		Gui::Session::View_handle //*
			_handle; //+ "bareWindow" or "undecoratedWindow" ?
			// + windowHandle !
};



//#pragma mark -


/*
class hog::MouseState
{
	// Pointing device's latest state retrieved from Genode.Input
	// (Used by: BView.GetMouse() et alia)

public:

		MouseState()
		:	curScreenWhere()
			,curButtons( 0 )
		{
		}
		  
public:  // !

		// current/last-known state, in Haiku API parlance
		BPoint curScreenWhere;
		int32  curButtons;
};
*/


#ifdef HAS_MICRO_WINDOW_MANAGER

class hog::Wm
{
	// A microscopic "window manager", so that our (Be-style) windows can be moved
	// and closed even though they interact only with Nitpicker (but not with Wm)
	// and draw their own window decorations.
	
public:
		Wm( BWindow & win );
		
		void HandleEvent(
			const Input::Event & ev,
		//	const hog::MouseState & state
			const BPoint curScreenWhere
		//	const uint32 curButtons
			);

private:

		BWindow & ourWin;
		
		bool isDragging;  // if true, we're moving the window with the below hot-spot
		BPoint trackWithHot;
};

#endif  // ~HAS_MICRO_WINDOW_MANAGER




class hog::InputBridger
{
public:
		InputBridger( BWindow & win );
		~InputBridger();
		
	//static int input_server_( void * data );
		void eventLoop();
		
		void handleInput();


private:  // Code

		void handleOneEvent(
			const Input::Event & ev
			);
		BMessage mouseDown( int32 button );
		BMessage mouseUp( int32 button );
		BMessage keyDownOrUp(
			const Input::Keycode genode_key,
			const bool latch_down );
		
	static int forwarding_to_pthread( void * bridger_uncast );


private:  // Data

		// Tx -- for pushing to PostMessage() and access to ServerWindow()
		BWindow & ourWin;
		
		// State --
		// double-click detection state
		bigtime_t latestClick;  // last time any button (left, right, etc) was pressed down, in support of mouseDown()
		// current state, previously in struct "MouseState":
		BPoint curScreenWhere;
		int32  curButtons;
		
		// distinct thread, to separate Genode-context and LibC-context code,
		// communicating through a mutex'ed std::queue:
		//
		thread_id threadId;
		Genode::Entrypoint localEp;  // Genode thread dedicated to the sigh
		// Rx -- let Genode push events to us
		Genode::Signal_handler<InputBridger>
			inputHandler;// { be_app->Env().ep(), *this, &InputBridger::handleInput };
		
		// shared between our <ep> Genode-thread and <forwarder> pthread
		//
		Genode::Blockade greenlightForwarder;
		Genode::Mutex mutexFwd;
		std::queue< Input::Event > eventsToForward;
		bool tearDownThread;  // set to <true> to signal function forwarding_to_pthread() to end its thread
		
#ifdef HAS_MICRO_WINDOW_MANAGER
		Wm winManager;
#endif  // ~HAS_MICRO_WINDOW_MANAGER
};



#endif // _WindowNitpickerPriv_H

